#!/bin/bash
SIMULATOR=$1

if [ -z "${SIMULATOR}" ]; then
	echo "Usage: $(basename $0) <simulator>"
	exit 0
fi

function Console()
{
	nc localhost 1234
}

printf "" > log.txt

BENCHDIRS="\
xps_timer/poll \
xps_timer/pwm \
xps_timer/gen \
xps_timer/cap \
s29gl256p/single_word_programming \
s29gl256p/unlock_bypass_word_programming \
s29gl256p/cfi_query \
s29gl256p/sector_erase \
s29gl256p/chip_erase \
s29gl256p/autoselect \
s29gl256p/unlock_bypass_sector_erase \
s29gl256p/write_buffer_programming \
s29gl256p/unlock_bypass_chip_erase \
xps_uart_lite/echo \
"

for BENCHDIR in ${BENCHDIRS}; do
	printf "Running ${BENCHDIR}..."
	printf "${BENCHDIR}:" >> log.txt
	if [ "${BENCHDIR}" = "xps_timer/pwm" ]; then
		${SIMULATOR} -c ${BENCHDIR}/sim_config.xml -s telnet.enable-negotiation=false &> ${BENCHDIR}/sim_out.txt &
		PID=$!
	else
		${SIMULATOR} -c ${BENCHDIR}/sim_config.xml -s telnet.enable-negotiation=false &> ${BENCHDIR}/sim_out.txt &
	fi
	sleep 1
	if [ "${BENCHDIR}" = "xps_uart_lite/echo" ]; then
		printf "line1\nline2\nline3\n\n" | Console > "${BENCHDIR}/console_out.txt"
	elif [ "${BENCHDIR}" = "xps_timer/pwm" ]; then
		Console > /dev/null
	else
		Console > "${BENCHDIR}/console_out.txt"
	fi
	if [ $? -ne 0 ]; then
		printf "FAIL\n" >> log.txt
	fi
	wait ${PID}
	if [ $? -ne 0 ]; then
		printf "FAIL\n" >> log.txt
	fi

	if [ "${BENCHDIR}" = "xps_timer/pwm" ]; then
		cat ${BENCHDIR}/sim_out.txt | grep "pwm-stub:" > ${BENCHDIR}/pwm_out.txt
		diff "${BENCHDIR}/pwm_out.ref.txt" "${BENCHDIR}/pwm_out.txt" &> /dev/null
		if [ $? -eq 0 ]; then
			printf "PASS\n" >> log.txt
		else
			printf "FAIL\n" >> log.txt
		fi
	else
		diff "${BENCHDIR}/console_out.ref.txt" "${BENCHDIR}/console_out.txt" &> /dev/null
		if [ $? -eq 0 ]; then
			printf "PASS\n" >> log.txt
		else
			printf "FAIL\n" >> log.txt
		fi
	fi
	printf "done\n"
done

cat log.txt
