/***************************************************************************
 *                                                                         *
 *   copyright        : (C) 2011 Commissariat a l'Energie Atomique (CEA)   *
 *   author           : Gilles Mouchard                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "flash.h"
#include "mmu.h"

#define FLASH_BASEADDR 0xfc000000UL
#define FLASH_BASE_PHYSICAL_ADDR 0xfc000000UL
#define FLASH_HIGH_PHYSICAL_ADDR 0xfdffffffUL

void flash_init(flash_t *flash)
{
	uint32_t byte_length = FLASH_HIGH_PHYSICAL_ADDR - FLASH_BASE_PHYSICAL_ADDR + 1;
	page_t page;
	
	page.valid = 1;
	page.ts = 0;
	page.base_effective_addr = FLASH_BASEADDR;
	page.tid = 0;
	page.base_physical_addr = FLASH_BASE_PHYSICAL_ADDR;
	page.size = get_page_size(byte_length);
	page.attr = ATTR_I | ATTR_SX | ATTR_SW | ATTR_SR;

	load_tlb(&page);

	flash->size = byte_length;
	flash->word_size = 2; // 16-bit word
	flash->sector_size = 128 * 1024; // 128 KB
	flash->data = (void *) FLASH_BASEADDR;
}
