/***************************************************************************
 *                                                                         *
 *   copyright        : (C) 2011 Commissariat a l'Energie Atomique (CEA)   *
 *   author           : Gilles Mouchard                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "intc.h"
#include "io.h"
#include "mmu.h"

#define INTC_BASEADDR 0x81800000UL
#define INTC_BASE_PHYSICAL_ADDR 0x81800000UL
#define INTC_HIGH_PHYSICAL_ADDR 0x8180ffffUL

intc_t *intc = 0;

void (*handler_table[32])();

void intc_init()
{
	if(intc) return;
	unsigned int i;
	uint32_t byte_length = INTC_HIGH_PHYSICAL_ADDR - INTC_BASE_PHYSICAL_ADDR + 1;
	page_t page;
	
	page.valid = 1;
	page.ts = 0;
	page.base_effective_addr = INTC_BASEADDR;
	page.tid = 0;
	page.base_physical_addr = INTC_BASE_PHYSICAL_ADDR;
	page.size = get_page_size(byte_length);
	page.attr = ATTR_I | ATTR_SX | ATTR_SW | ATTR_SR;

	load_tlb(&page);

	intc = (intc_t *) INTC_BASEADDR;
	
	write_big32(&intc->ier, 0);   // disable all IRQs
	write_big32(&intc->mer, 0x3); // MER[HIE]=1 MER[ME]=1
	
	for(i = 0; i < 32; i++) handler_table[i] = 0;
}

void register_irq_handler(unsigned int irq, void (*handler)())
{
	handler_table[irq] = handler;
}

void enable_irq(unsigned int irq)
{
	write_big32(&intc->ier, read_big32(&intc->ier) | (1 << irq)); // enable IRQ
}

void disable_irq(unsigned int irq)
{
	write_big32(&intc->ier, read_big32(&intc->ier) & ~(1 << irq)); // disable IRQ
}

void external_input_isr()
{
	if(intc)
	{
		uint32_t ivr = read_big32(&intc->ivr);
		
		if(ivr < 32)
		{
			write_big32(&intc->iar, 1 << ivr); // acknowlegde interrupt in interrupt controller
			if(handler_table[ivr])
			{
				handler_table[ivr]();
			}
		}
	}
}
