#include "ppc44x_regs.h"

#define IVOR_CRITICAL_INPUT 0
#define IVOR_MACHINE_CHECK 1
#define IVOR_DATA_STORAGE 2
#define IVOR_INSTRUCTION_STORAGE 3
#define IVOR_EXTERNAL_INPUT 4
#define IVOR_ALIGNMENT 5
#define IVOR_PROGRAM 6
#define IVOR_FLOATING_POINT_UNAVAILABLE 7
#define IVOR_SYSTEM_CALL 8
#define IVOR_AUXILIARY_PROCESSOR_UNAVAILABLE 9
#define IVOR_DECREMENTER 10
#define IVOR_FIXED_INTERVAL_TIMER 11
#define IVOR_WATCHDOG_TIMER 12
#define IVOR_DATA_TLB_ERROR 13
#define IVOR_INSTRUCTION_TLB_ERROR 14
#define IVOR_DEBUG 15

#define SET_IVOR(ivor_num, isr_label) \
	li r0, isr_label@l; \
	mtspr IVOR##ivor_num, r0; \
	sync

#define EXCEPTION_PROLOG \
	mtspr SPRG0, r0;  /* save r0 */ \
	mtspr SPRG1, r1;  /* save r1 */ \
	lis r1, 0x07ff; \
	ori r1, r1, 0xfffc;    /* stack pointer <- 0x7fffffc (below 128 MB) */ \
	subi r1, r1, 136; /* reserve space for saving 30 * 32-bit registers + 4 */ \
	stmw r2, 0(r1);     /* save r2 -> r31 on stack */ \
	mfxer r0; \
	stw r0, 120(r1);   /* save XER */ \
	mfcr r0; \
	stw r0, 124(r1);   /* save CR */ \
	mflr r0; \
	stw r0, 128(r1);   /* save LR */ \
	mfctr r0; \
	stw r0, 132(r1)   /* save CTR */

#define EXCEPTION_HANDLING \
	bl exception_handler

#define EXCEPTION_EPILOG \
	lwz r0, 132(r1); \
	mtctr r0; /* restore CTR */ \
	lwz r0, 128(r1); \
	mtlr r0; /* restore LR */ \
	lwz r0, 124(r1); \
	mtcr r0; /* restore CR */ \
	lwz r0, 120(r1); \
	mtxer r0; /* restore XER */ \
	lmw r2, 0(r1);     /* restore r2 -> r31 from stack */ \
	mfspr r1, SPRG1;  /* restore r1 */ \
	mfspr r0, SPRG0  /* restore r0 */

	.global external_input_isr

critical_input:
	b critical_input
	rfi

machine_check:
	b machine_check
	rfi

data_storage:
	b data_storage
	rfi

instruction_storage:
	b instruction_storage
	rfi

external_input:
	EXCEPTION_PROLOG
	subi r1, r1, 8 /* reserve space for LR save */
	bl external_input_isr
	addi r1, r1, 8
	EXCEPTION_EPILOG
	rfi

alignment:
	b alignment
	rfi

program:
	b program
	rfi

floating_point_unavailable:
	b floating_point_unavailable
	rfi

system_call:
	b system_call
	rfi

auxiliary_processor_unavailable:
	b auxiliary_processor_unavailable
	rfi

decrementer:
	b decrementer
	rfi

fixed_interval_timer:
	b fixed_interval_timer
	rfi

watchdog_timer:
	b watchdog_timer
	rfi

data_tlb_error:
	b data_tlb_error
	rfi

instruction_tlb_error:
	b instruction_tlb_error
	rfi

debug:
	b debug

	.global _init_interrupts
_init_interrupts:
	subi r1, r1, 4
	stw r0, 0(r1)
	li r0, 0
	mtspr IVPR, r0
	SET_IVOR(0, critical_input)
	SET_IVOR(1, machine_check)
	SET_IVOR(2, data_storage)
	SET_IVOR(3, instruction_storage)
	SET_IVOR(4, external_input)
	SET_IVOR(5, alignment)
	SET_IVOR(6, program)
	SET_IVOR(7, floating_point_unavailable)
	SET_IVOR(8, system_call)
	SET_IVOR(9, auxiliary_processor_unavailable)
	SET_IVOR(10, decrementer)
	SET_IVOR(11, fixed_interval_timer)
	SET_IVOR(12, watchdog_timer)
	SET_IVOR(13, data_tlb_error)
	SET_IVOR(14, instruction_tlb_error)
	SET_IVOR(15, debug)
	mfmsr r0
	ori r0, r0, 0x8000
	mtmsr r0              /* MSR[EE]=1: enable external interrupts */
	lwz r0, 0(r1)
	addi r1, r1, 4
	blr
