/***************************************************************************
 *                                                                         *
 *   copyright        : (C) 2011 Commissariat a l'Energie Atomique (CEA)   *
 *   author           : Gilles Mouchard                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "timer.h"
#include "intc.h"
#include "io.h"
#include "mmu.h"

#define TIMER_BASEADDR 0x83c00000UL
#define TIMER_BASE_PHYSICAL_ADDR 0x83c00000UL
#define TIMER_HIGH_PHYSICAL_ADDR 0x83c0ffffUL

timer_t *timer_init()
{
	uint32_t byte_length = TIMER_HIGH_PHYSICAL_ADDR - TIMER_BASE_PHYSICAL_ADDR + 1;
	page_t page;
	timer_t *timer;
	
	page.valid = 1;
	page.ts = 0;
	page.base_effective_addr = TIMER_BASEADDR;
	page.tid = 0;
	page.base_physical_addr = TIMER_BASE_PHYSICAL_ADDR;
	page.size = get_page_size(byte_length);
	page.attr = ATTR_I | ATTR_SX | ATTR_SW | ATTR_SR;

	load_tlb(&page);

	timer = (timer_t *) TIMER_BASEADDR;
	
	// disable everything
	write_big32(&timer->tcsr0, 0); 
	write_big32(&timer->tcsr1, 0);
	
	return timer;
}
