/***************************************************************************
 *                                                                         *
 *   copyright        : (C) 2011 Commissariat a l'Energie Atomique (CEA)   *
 *   author           : Gilles Mouchard                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "flash.h"
#include "intc.h"
#include "uart_lite.h"
#include "io.h"
#include <inttypes.h>

uint16_t manuf_id, dev_id1, dev_id2, dev_id3, sector_protect0;

void halt() __attribute__((noinline));

void halt()
{
	do {} while(1);
}

int main()
{
	flash_t flash;
	intc_init();
	flash_init(&flash);
	uart_lite_init();
	
	/* Auto Select Entry */
	PrintString("Write unlock cycle 1\n");
	write_little16((uint16_t *) flash.data + 0x555, 0x00AA); /* write unlock cycle 1 */
	PrintString("Write unlock cycle 2\n");
	write_little16((uint16_t *) flash.data + 0x2AA, 0x0055); /* write unlock cycle 2 */
	PrintString("Write autoselect command\n");
	write_little16((uint16_t *) flash.data + 0x555, 0x0090); /* write autoselect command */
	
	/* multiple reads can be performed after entry */
	manuf_id = read_little16((uint16_t *) flash.data + MANUF_ID); /* read manuf. id */
	PrintString("Manufacturer ID: ");
	PrintHex16(manuf_id);
	PrintString("\n");
	
	dev_id1 = read_little16((uint16_t *) flash.data + DEV_ID1); /* read dev. id word 1 */
	PrintString("Device ID word 1: ");
	PrintHex16(dev_id1);
	PrintString("\n");
	
	dev_id2 = read_little16((uint16_t *) flash.data + DEV_ID2); /* read dev. id word 2 */
	PrintString("Device ID word 2: ");
	PrintHex16(dev_id2);
	PrintString("\n");
	
	dev_id3 = read_little16((uint16_t *) flash.data + DEV_ID3); /* read dev. id word 3 */
	PrintString("Device ID word 3: ");
	PrintHex16(dev_id3);
	PrintString("\n");
	
	PrintString("Protection status (U=unprotected/P=protected): ");
	uint32_t sector;
	uint32_t num_sectors = flash.size / flash.sector_size;
	for(sector = 0; sector < num_sectors; sector++)
	{
		uint16_t sector_protect = read_little16((uint16_t *) flash.data + ((sector << 8) | SECTOR_PROTECT_VERIFY)); /* read sector protection status */
		PrintString(sector_protect ? "P" : "U");
	}
	PrintString("\n");
	/* Autoselect exit */
	PrintString("Exit autoselect\n");
	write_little16((uint16_t *) flash.data + 0x000, 0x00F0); /* exit autoselect (write reset command) */
	
	PrintString("Test is finished\n");
	halt();
	return 0;
}
