/***************************************************************************
 *                                                                         *
 *   copyright        : (C) 2011 Commissariat a l'Energie Atomique (CEA)   *
 *   author           : Gilles Mouchard                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "flash.h"
#include "io.h"
#include "uart_lite.h"
#include "intc.h"
#include <inttypes.h>

void halt() __attribute__((noinline));

void halt()
{
	do {} while(1);
}

int main()
{
	uint32_t sector;
	uint16_t data;
	flash_t flash;

	intc_init();
	flash_init(&flash);
	uart_lite_init();

	PrintString("Write unlock cycle 1\n");
	write_little16((uint16_t *) flash.data + 0x555, 0x00AA);  /* write unlock cycle 1 */
	PrintString("Write unlock cycle 2\n");
	write_little16((uint16_t *) flash.data + 0x2AA, 0x0055);  /* write unlock cycle 2 */
	PrintString("Write unlock bypass command\n");
	write_little16((uint16_t *) flash.data + 0x555, 0x0020);  /* write unlock bypass command */
	
	uint32_t num_sectors = flash.size / flash.sector_size;
	for(sector = 0; sector < num_sectors; sector++)
	{
		PrintString("Erasing sector #");
		PrintUInt32(sector);
		PrintString(" of ");
		PrintUInt32(num_sectors);
		PrintString("\n");
		PrintString("Write setup command\n");
		write_little16((uint16_t *) flash.data + 0x555, 0x0080);  /* write setup command */
		PrintString("Write sector erase command for sector #");
		PrintUInt32(sector);
		PrintString("\n");
		write_little16((uint16_t *) flash.data + ((sector * flash.sector_size) / flash.word_size), 0x0030);     /* write sector erase command */
		
		PrintString("Verifying sector: ");
		
		uint32_t offset;
		uint32_t sector_offset = sector * flash.sector_size;
		for(offset = 0; offset < flash.sector_size; offset += flash.word_size)
		{
			do
			{
				data = read_little16((uint16_t *) flash.data + ((sector_offset + offset) / flash.word_size));
			}
			while(data != 0xffff);
		}
		PrintString("OK\n");
	}
	/* exit unlock bypass */
	PrintString("Exit unlock bypass\n");
	write_little16((uint16_t *) flash.data + 0x000, 0x0090);  
	write_little16((uint16_t *) flash.data + 0x000, 0x0000);
	PrintString("Test is finished\n");
	halt();
	return 0;
}
