/***************************************************************************
 *                                                                         *
 *   copyright        : (C) 2011 Commissariat a l'Energie Atomique (CEA)   *
 *   author           : Gilles Mouchard                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "flash.h"
#include "io.h"
#include "uart_lite.h"
#include "intc.h"
#include <inttypes.h>

void halt() __attribute__((noinline));

void halt()
{
	do {} while(1);
}

int main()
{
	unsigned int i = 0;
	uint32_t pa = 0x123456;
	unsigned int wc = 3;
	uint16_t data_to_program[4] = { 0x4321, 0x8765, 0x9087, 0x3852 };
	uint16_t programmed_data[4] = { 0, 0, 0, 0 };
	flash_t flash;
	
	intc_init();
	flash_init(&flash);
	uart_lite_init();
	
	PrintString("Write unlock cycle 1\n");
	write_little16((uint16_t *) flash.data + 0x555, 0x00AA);  /* write unlock cycle 1 */
	PrintString("Write unlock cycle 2\n");
	write_little16((uint16_t *) flash.data + 0x2AA, 0x0055);  /* write unlock cycle 2 */
	PrintString("Write write buffer load command\n");
	write_little16((uint16_t *) flash.data + pa, 0x0025);  /* write write buffer load command */
	PrintString("Write write word count (minus 1)\n");
	write_little16((uint16_t *) flash.data + pa, wc);  /* write write word count (minus 1) */
	
	do
	{
		PrintString("Write data word #");
		PrintUInt32(i);
		PrintString(" (");
		PrintHex16(data_to_program[i]);
		PrintString(")\n");
		write_little16((uint16_t *) flash.data + pa + i, data_to_program[i]);       /* write data to be programmed */
		i++;
	}
	while(wc--);
	PrintString("Write confirm command\n");
	write_little16((uint16_t *) flash.data + pa, 0x0029);  /* write confirm command */
	
	i = 0;
	wc = 3;
	do
	{
		PrintString("Re-reading and verifying word #");
		PrintUInt32(i);
		PrintString(": ");
		programmed_data[i] = read_little16((uint16_t *) flash.data + pa + i);
		PrintString((programmed_data[i] == data_to_program[i]) ? "OK" : "FAILED");
		PrintString(" (");
		PrintHex16(programmed_data[i]);
		PrintString(")\n");
		i++;
	}
	while(wc--);
	PrintString("Test is finished\n");
	halt();
	return 0;
}
