/***************************************************************************
 *                                                                         *
 *   copyright        : (C) 2011 Commissariat a l'Energie Atomique (CEA)   *
 *   author           : Gilles Mouchard                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "timer.h"
#include "io.h"
#include "uart_lite.h"
#include "intc.h"
#include <inttypes.h>



timer_t *timer = 0;
int tick = 0;
unsigned int tick_count = 0;

void timer_irq_handler()
{
	if(timer)
	{
		if(read_big32(&timer->tcsr0) & TCSR0_T0INT)
		{
			write_big32(&timer->tcsr0, read_big32(&timer->tcsr0) | TCSR0_T0INT); // acknowlegde interrupt in timer
			tick_count++;
			tick = 1;
		}
	}
}

void halt() __attribute__((noinline));

void halt()
{
	do {} while(1);
}

#define TIMER_PERIOD 20000  /* 100 us */

int main()
{
	intc_init();
	uart_lite_init();
	
	timer = timer_init(); // Map in memory and initialize timer
	register_irq_handler(TIMER_IRQ, timer_irq_handler);
	
	write_big32(&timer->tlr0, TIMER_PERIOD - 2); // TIMING_INTERVAL= TIMER_PERIOD * TIMER_CYCLE_TIME

	write_big32(&timer->tcsr0, TCSR0_LOAD0); // Move TLR0 into TCR0
	write_big32(&timer->tcsr0, TCSR0_ENT0 | TCSR0_UDT0 | TCSR0_ENIT0 | TCSR0_ARHT0); // enable timer 0, down counter, generate mode, enable interrupt signal, autoreload timer counter

	enable_irq(TIMER_IRQ);
	
	unsigned int new_tick_count = 0;
	unsigned int last_tick_count = 0;
	do
	{
		new_tick_count = tick_count;
		if(last_tick_count != new_tick_count)
		{
			PrintString("Tick: ");
			PrintUInt32(new_tick_count * (TIMER_PERIOD * TIMER_CYCLE_TIME / 1000));
			PrintString(" us\n");
			last_tick_count = new_tick_count;
		}
	}
	while(last_tick_count < 100);
	
	write_big32(&timer->tcsr0, 0); // disable everything
	disable_irq(TIMER_IRQ);
	PrintString("Test is finished\n");
	halt();
	return 0;
}
